--[[ LibNearFar
     NEAR?
     FAR!

]]--

local addoninfo, nf = ...
Library = Library or {}
Library.LibNearFar = nf

nf.printf = Library.printf.printf

function nf.distance_calc(unit1, unit2)
  local dx = unit1.coordX - unit2.coordX
  local dy = unit1.coordY - unit2.coordY
  local dz = unit1.coordZ - unit2.coordZ
  return math.sqrt(dx * dx + dy * dy + dz * dz)
end

function nf.range(unit1, unit2, default)
  if type(unit1) == 'string' then
    unit1 = Inspect.Unit.Detail(unit1)
  end
  if type(unit2) == 'string' then
    unit2 = Inspect.Unit.Detail(unit2)
  end
  if type(unit1) ~= 'table' or type(unit2) ~= 'table' then
    return default
  end
  if unit1.coordX and unit1.coordY and unit1.coordZ and
     unit2.coordX and unit2.coordY and unit2.coordZ then
    local status, value = pcall(nf.distance_calc, unit1, unit2)
    if status then
      return value - (unit1.radius or 0) - (unit2.radius or 0)
    end
  end
  return default
end


function nf.distance(unit1, unit2, default)
  if type(unit1) == 'string' then
    unit1 = Inspect.Unit.Detail(unit1)
  end
  if type(unit2) == 'string' then
    unit2 = Inspect.Unit.Detail(unit2)
  end
  if type(unit1) ~= 'table' or type(unit2) ~= 'table' then
    return default
  end
  if unit1.coordX and unit1.coordY and unit1.coordZ and
     unit2.coordX and unit2.coordY and unit2.coordZ then
    local status, value = pcall(nf.distance_calc, unit1, unit2)
    if status then
      return value
    end
  end
  return default
end

function nf.prettyrange(unit_1_or_d, unit2, default)
  local d = unit1_or_d
  if not d or type(d) ~= 'number' then
    d = nf.range(unit1_or_d, unit2, default)
    if not d or type(d) ~= 'number' then
      return default
    end
  end
  return nf.prettydistance(d, nil, default)
end

function nf.prettydistance(unit1_or_d, unit2, default)
  local d = unit1_or_d
  if not d or type(d) ~= 'number' then
    d = nf.distance(unit1_or_d, unit2, default)
    if not d or type(d) ~= 'number' then
      return default
    end
  end
  if d >= 1000 then
    return string.format("%.3fkm", d / 1000)
  elseif d >= 100 then
    return string.format("%.1fm", d)
  else
    return string.format("%.2fm", d)
  end
end

